define(['jquery'], function (jQuery) {
    'use strict';

    (function ($) {
        var defaults = {
            places: 7,
            decimalPlaces: 2,
            allowEmpty: false,
            allowMinus: true,
            type: 'currency', // available values: currency, percentage, int, any
            // type 'any' is used for max length only
            clearButton: false
        };
        var methods = {
            _options: {},
            init : function (receivedOptions) {
                if(this.length == 0) {
                    return false;
                }
                var options = _.extend({decimalSeparator: $.global.culture.numberFormat['.']}, defaults, receivedOptions || {});

                methods.setOptions(this, options);
                this.data('type', options.type);
                methods.changeRegexp(options);

                if(!this.closest('.input-format').length) {
                    methods.prepareHtml(this);
                }
                methods.convertAndSetValue(this, options.startValue ? options.startValue : this.val());
                methods.setValidation(this);

                if(options.clearButton){
                    var $parent = $(this).parent().parent();
                    $parent.prepend($('<div class="input-format-clear-icon button"></div>'));
                    $parent.on('touchend.inputFormat', '.input-format-clear-icon', function() {
                        methods.setValue.call(this, options.allowEmpty ? '' : 0);
                    }.bind(this));
                }

                return this;
            },

            setOptions: function (ctx, options) {
                methods._options[ctx[0].id] = options;
            },

            getOptions: function (ctx) {
                var htmlElement = ctx[0];
                var id = htmlElement ? htmlElement.id : "";
                return methods._options[id];
            },

            prepareHtml: function (ctx) {
                var $wrapper = $('<div class="input-format"></div>');
                $wrapper.css({
                    font: ctx.css('font'),
                    height: ctx.css('height'),
                    lineHeight: ctx.css('height'),
                    textAlign: ctx.css('text-align'),
                    color: ctx.css('color'),
                    float: ctx.css('float'),
                    textTransform: ctx.css('text-transform'),
                    margin: ctx.css('margin'),
                    paddingLeft: ctx.css('padding-left'),
                    paddingRight: ctx.css('padding-right'),
                    background: ctx.css('background')
                });

                ctx.wrap($wrapper);
                // set wrapper width in percents
                ctx.parent()
                    .hide()
                    .css({width: ctx.css('width')})
                    .show();

                ctx.css({
                    margin: 0,
                    width: '100%'
                });
                ctx.after('<span class="val"/>');
            },

            setValidation: function (ctx) {
                var options = methods.getOptions(ctx);
                var breakInputEvent = false;
                ctx.on({
                    'keypress.inputFormat': function(e) {
                        if(e.which == 45) {
                            breakInputEvent = true;
                        }
                    },
                    'input.inputFormat': function () {
                        var val = this.value,
                            lastVal = $(this).data('lastVal');
                        if(options.allowMinus) {
                            if(breakInputEvent || val.replace(/[^-]/g, '').length > lastVal.replace(/[^-]/g, '').length) {
                                breakInputEvent = false;
                                methods.changeSign.call(ctx, lastVal);
                                $(this).data('lastVal', this.value);
                                return false;
                            }
                        }

                        $(this).closest('.form-group').removeClass('error');

                        if (val === '') {
                            if ($(this).is('[type=number]')) {
                                if (lastVal.replace(/[^-]/g, '').length) {
                                    this.value = val;
                                }
                            } else { // type=text and tel
                                this.value = val;
                                $(this).data('lastVal', this.value);
                            }
                        } else {
                            if (val === '-') {
                                this.value = '';
                                return false;
                            }
                            if (!methods.isValidString(val, options)) {
                                this.value = lastVal;
                            } else {
                                $(this).data('lastVal', this.value);
                            }
                        }
                    },
                    'focus.inputFormat': function () {
                        methods.setSelection(this);
                        $(this).data('lastVal', this.value);
                    },
                    'blur.inputFormat': function () {
                        // TODO figure out
                        // this fixes MSI-671 ?
                        // but creates MSI-706:
                        $(this).trigger('change');
                        //
                        if(!methods.getOptions(ctx).allowEmpty && this.value === '') {
                            $(this).val(0);
                        }
                        $(this).siblings('.val').text(
                            methods.getFormattedNumber(
                                methods.getClearNumber(this.value, options), options.type
                            )
                        );
                    }
                });
            },

            isValidString: function (str, options) {
                if (str === '' && options.allowEmpty) return true;
                else return new RegExp(options.regex).test(str);
            },

            setSelection: function(ctx) {
                if(!methods.getOptions($(ctx)).allowEmpty && parseInt($(ctx).val()) == 0) {
                    $(ctx).val('');
                } else if ($(ctx).is('[readonly]')) {
                    e.preventDefault();
                    $(ctx).blur();
                } else {
                    setTimeout(function() {
                        try {
                            ctx.setSelectionRange(0, 9999);
                        } catch (ex){
                            // do nothing
                        }
                    }, 0);
                }
            },

            getClearNumber: function (value, options) {
                if (!options) return 0;
                if(value === '') {
                    if (options.allowEmpty) {
                        return '';
                    }
                    return $.global.parseFloat('0');
                }
                if (options.type == 'any') {
                    return value;
                }
                return $.global.parseFloat(value.toString());
            },

            getFormattedNumber: function (value, type) {
                if(value === '') {
                    return '';
                }
                switch (type) {
                    case 'currency':
                        return $.global.format(+value, 'c');
                    case 'percentage':
                        return $.global.format(+value / 100, 'p');
                    case 'int':
                        return $.global.format(+value, 'd');
                    case 'any':
                        return value;
                }
            },

            convertAndSetValue: function (ctx, newValue) {
                var options = methods.getOptions(ctx),
                    val = methods.getClearNumber(newValue, options);
                ctx.val(val);
                ctx.siblings('.val').text(methods.getFormattedNumber(val, ctx.data('type')));
            },

            changeType: function (newType) {
                var options = methods.getOptions(this);
                options.type = newType;
                this.data('type', options.type);
                methods.changeRegexp(options);
                methods.convertAndSetValue(this, this.val());
            },

            changeSign: function(newValue) {
                methods.setValue.call(this, -newValue);
            },

            changeRegexp: function (options) {
                switch(options.type) {
                    case 'currency':
                    case 'percentage':
                        options.regex = '(?=.)^(' + (options.allowMinus ? '-?' : '');
                        options.regex += '(([1-9][0-9]{0,' + (options.places - 1) + '})|0))?';
                        options.regex += '(\\' + options.decimalSeparator + '([0-9]{1,' + options.decimalPlaces + '})?)?$';
                        break;
                    case 'int':
                        options.regex = '^([1-9][0-9]{0,' + (options.places - 1) + '}|0)$';
                        break;
                    case 'any':
                        options.regex = '^(\(.*){0,' + options.places + '})$';
                        break;
                }
            },

            getValue: function () {
                if (this.length === 0) return 0;
                return methods.getClearNumber(this.val(), methods.getOptions(this));
            },

            getType: function () {
                return methods.getOptions(this).type;
            },

            setValue: function (newValue) {
                var options = methods.getOptions(this);
                if(options.type != 'any' && newValue !== '') {
                    newValue = +(parseFloat(newValue).toFixed(2));
                }
                if (methods.isValidString(newValue, options)) {
                    methods.convertAndSetValue(this, newValue);
                    this.trigger('change');
                }
                return this;
            },

            destroy: function () {
                // this - all selected inputs
                _.each(this, function (input) {
                    delete methods._options[input.id];
                    $(this).off('.inputFormat');
                });
            }
        };

        $.fn.inputFormat = function(methodOrOptions) {
            if ( methods[methodOrOptions] ) {
                return methods[ methodOrOptions ].apply( this, Array.prototype.slice.call( arguments, 1 ));
            } else if ( typeof methodOrOptions === 'object' || ! methodOrOptions ) {
                // Default to "init"
                return methods.init.apply( this, arguments );
            } else {
                $.error( 'Method ' +  methodOrOptions + ' does not exist on jQuery.inputFormat' );
            }
        };
    })(jQuery);
});