define([
    'common/base-view',
    'views/desktop',
    'data/desktops',
    'text!../../templates/smart-center.html',
    'swipeview',
    'jqueryQuickfit'
], function (BaseView, DesktopView, desktopsList, smartCenterTemplate) {
    'use strict';
    return BaseView.extend({
        el: '.smart-center',
        template: _.template(smartCenterTemplate),
        optionTemplate: _.template('<option value="<%- value %>"><%- label %></option>'),
        firstTimeRefresh: true,
        activeIndex: 0,

        initialize: function () {
            this.rotateEventHandler = this.rotateEvent.bind(this);

            this.listenTo(desktopsList, 'add', this.desktopsDataChanged.bind(this, 'add'));
            this.listenTo(desktopsList, 'remove', this.desktopsDataChanged.bind(this, 'remove'));
            this.listenTo(desktopsList, 'reset', this.desktopsDataChanged.bind(this, 'reset'));
        },

        events: {
            'change #desktops-select': 'desktopSelect'
        },

        refresh: function () {
            Backbone.history.loadUrl(Backbone.history.fragment);
        },

        desktopsDataChanged: function (type, desktop, list) {
            switch (type) {
                case 'add':
                    this.$('#desktops-select').append(this.optionTemplate({
                        label: desktop.get('displayName'),
                        value: desktop.get('id')
                    }));
                    break;
                case 'remove':
                    this.$('#desktops-select [value="' + desktop.get('id') + '"]').remove();
                    break;
                case 'reset':
                    this.$('#desktops-select').empty();
                    break;
            }

            this.$('#desktops-select').toggleClass('hidden', desktopsList.length == 0);
            this.refreshSwipeView(this.firstTimeRefresh);
            this.setSwipeControls();
        },

        render: function () {
            this.$el.html(this.template());

            this.swipeview = new SwipeView('.swipe', {numberOfPages: desktopsList.length});
            this.swipeview.onTouchStart(function () {
                var swipe = this.swipeview;
                var currectPage = swipe.masterPages[swipe.currentMasterPage];
                var scroll = $(':first-child', currectPage).data('scroll');
                swipe.initiated = scroll.enabled && this.$('.loading').size() == 0;
            }.bind(this));
            this.swipeview.onFlip(function () {
                var swipe = this.swipeview;

                _.each(swipe.masterPages, function (master, index) {
                    var $master = $(master);
                    var upcoming = master.dataset.upcomingPageIndex;
                    if (desktopsList.length > 0) {
                        var desktop = desktopsList.at(upcoming);
                        if (desktop && !$master.is(':has(#' + desktop.get('id') + ')')) {
                            if ($master.data('view')) {
                                $master.data('view').close();
                            }
                            var view = new DesktopView({model: desktop});
                            $master.html(view.render().$el);
                            $master.data('view', view);
                        }
                        this.setTileTextHeight();
                    } else {
                        $master.html('<p style="padding:0 5%;color:#fff;text-align:center;">No Smart Center Groups have been selected for this user</p>');
                    }
                }.bind(this));

                var desktopId = $('.desktop', swipe.masterPages[swipe.currentMasterPage]).attr('id');
                if (desktopId) {
                    this.$('#desktops-select').val(desktopId);
                    this.$('#desktops-nav .selected').removeClass('selected');
                    this.$('#desktops-nav span[data-id=' + desktopId + ']').addClass('selected');
                    this.activeIndex = this.$('#desktops-nav .selected').index();
                    if ($('.details-wrapper').length == 0) {
                        Backbone.history.navigate('desktops/' + desktopId, {trigger: true, replace: true});
                    }
                }
            }.bind(this));
            this.swipeview.goToPage(0);

            return this;
        },

        show: function () {
            BaseView.prototype.show.call(this);
            this.bindEvents();

            this.$('#desktops-select').toggleClass('hidden', desktopsList.length == 0);
            this.refreshSwipeView();
            this.setSwipeControls();
            this.setTileTextHeight();
        },

        hide: function () {
            this.unbindEvents();
            if (this.swipeview) {
                $(this.swipeview.masterPages).each(function () {
                    $(this).data('view') && $(this).data('view').close();
                    $(this).removeData('view');
                });
            }
            BaseView.prototype.hide.call(this);
        },

        close: function () {
            this.unbindEvents();

            delete this.rotateEventHandler;

            _.each(this.swipeview.masterPages, function (master) {
                var view = $(master).data('view');
                if (view) {
                    view.close();
                }
            });

            this.swipeview.destroy();
            delete this.swipeview;

            _.each(this.childViews, function (view) {
                view.close();
            });
            delete this.childViews;

            BaseView.prototype.close.call(this);
        },

        refreshSwipeView: function (goto) {
            if (this.swipeview) {
                this.swipeview.refreshSize();
                this.swipeview.updatePageCount(desktopsList.length || 1);
                if (goto) {
                    this.swipeview.goToPage(desktopsList.length ? this.$('#desktops-nav .selected').index() : 0);
                }
            }
            this.firstTimeRefresh = false;
        },

        setSwipeControls: function () {
            var $nav = this.$('#desktops-nav').empty();
            desktopsList.each(function (desktop) {
                $nav.append('<span data-id="' + desktop.get('id') + '"></span>');
            });
            $nav.find('[data-id=' + this.$('#desktops-select').val() + ']').addClass('selected');
        },

        desktopSelect: function (e) {
            this.swipeview.goToPage($(':selected', e.target).index());
        },

        bindEvents: function () {
            this.unbindEvents();
            $(document).on('touchmove', preventTouchForIScroll);
            $(window).on('orientationHandlerCallback', this.rotateEventHandler);
        },

        unbindEvents: function () {
            $(document).off('touchmove', preventTouchForIScroll);
            $(window).off('orientationHandlerCallback', this.rotateEventHandler);
        },

        setTileTextHeight: function () {
            window.maxFontSizeTileText = $('.device.tablet').is(':visible') && $('.device.portrait').is(':visible') ? 52 : 29;
            this.$('.tile-text').quickfit({
                min: minFontSizeTileText,
                max: maxFontSizeTileText,
                truncate: true
            });
        },

        rotateEvent: function () {
            this.refreshSwipeView(true);
            this.setTileTextHeight();
        }

    });
});
