define([
    'jquery',
    'moment',
    'chart',
    'common/base-view',
    'data/consts',
    'text!../../templates/sales-content.html',
    'text!../../templates/sales-chart.html',
    'bridge'
], function ($, moment, Chart, BaseView, Consts, salesContentTemplate, salesChartTemplate) {
    'use strict';

    var SalesChart = Backbone.Model.extend({
        defaults: function () {
            return {
                "title": "",
                "metricType": "",
                "currentValue": "",
                "previousValue": "",
                "isIncreased": true,
                "background": "#8246c2",
                "metricIntervals": [],
                "labels": [],
                "data": []
            }
        }
    });

    /**
     * Collection that contains all charts in sales
     * @param {Object} model Backbone model
     * */
    var SalesCollection = Backbone.Collection.extend({
        model: SalesChart
    });

    var salesPeriod = 'Today';

    var salesCollection = new SalesCollection();

    var SalesChartView = BaseView.extend({
        tagName: 'div',
        className: 'sales-tile',
        template: _.template(salesChartTemplate),
        chartOptions: {
            animation: false,
            showTooltips: false,
            scaleGridLineColor: "rgba(48,48,48,1)",
            // responsive: true,
            // maintainAspectRatio: false,
			scaleFontSize: 10,
        },
        chart: undefined,

        render: function () {
            this.$el.html(this.template(this.model.toJSON()));
            this.$el.children('div').css('background-color', this.model.get('background'));
            return this;
        },

        show: function () {
            var value = this.model.get('currentValue');
            if (this.model.get('metricType') == 5 || this.model.get('metricType') == 6) {
                value = $.global.format(value / 100, 'p');
            } else {
                value = $.global.format(value, 'c');
            }
            this.$el.find('.sales-tile-text').html(value);

            var changeAmount = this.model.get('currentValue') - this.model.get('previousValue');
            var triangleType = ''; // if equal triangle type would be "empty"
            if (changeAmount > 0) {
                triangleType = 'up'
            } else if (changeAmount < 0) {
                triangleType = 'down'
            }
            this.$el.find('.sales-triangle').removeClass('up down').addClass(triangleType);

            var metricIntervals = this.model.get('metricIntervals').reverse(),
                dataSet = _.pluck(metricIntervals, 'value'),
                startDatesRaw = _.pluck(metricIntervals, 'startDate'),
                endDatesRaw = _.pluck(metricIntervals, 'endDate'),
                labels = [];

            switch (salesPeriod) {
                case 'Today':
                    labels = _.map(startDatesRaw, function (d) {
                        return moment(d).format('L').substr(0, 5);
                    });
                    break;
                case 'Last7Days':
                    labels = _.map(startDatesRaw, function (d) {
                        return moment(d).format('ddd');
                    });
                    break;
                case 'ThisMonth':
                    labels = _.map(startDatesRaw, function (d, i) {
                        return moment(d).format('DD') + '-' + moment(endDatesRaw[i]).format('DD');
                    });
                    break;
            }

            this.model.set('labels', labels);
            this.model.set('data', dataSet);

            var chartData = {
                labels: labels,
                datasets: [
                    {
                        label: "",
                        fillColor: "rgba(160,195,90,0.5)",
                        strokeColor: "rgba(160,195,90,0.8)",
                        data: dataSet
                    }
                ]
            };

            if (this.chart !== undefined) {
                this.chart.destroy();
                this.chart = undefined;
            }

            var canvasContext = this.$el.find('canvas').get(0).getContext("2d");

            this.chart = new Chart(canvasContext).Bar(chartData, this.chartOptions);
        }
    });

    return BaseView.extend({
        tagName: 'div',
        el: '.sales',
        template: _.template(salesContentTemplate),
        scroll: '',
        charts: [],

        initialize: function () {
            this.listenTo(salesCollection, 'add', this.renderMetric);
            this.listenTo(salesCollection, 'reset', this.emptyElement);
            this.rotateEventHandler = this.rotateEvent.bind(this);
        },

        events: {
            'touchstart .sales-header-menu li': 'changeScopeTablet',
            'change .sales-header-menu.mobile': 'changeScopeMobile'
        },

        render: function () {
            this.$el.html(this.template());
            this.initializeIScroll();
            return this;
        },

        renderMetric: function (item) {
            var chartView = new SalesChartView({model: item}).render();
            this.$el.find('.sales-tiles').append(chartView.$el);
            chartView.show();
            this.charts.push(chartView);
        },

        emptyElement: function () {
            this.$el.find('.sales-tiles').empty();
            _.each(this.charts, function (item, index) {
                item.close();
            }, this);
            this.charts = [];
        },

        initializeIScroll: function () {
            this.scroll = new IScroll('.sales .sales-scroll', {
                tap: true
            });
            this.scroll.on('beforeScrollStart', function () {
                this.refresh();
            });
        },

        show: function () {
            //$(window).on("orientationHandlerCallback", this.rotateEventHandler);
            $(document).on('touchmove', preventTouchForIScroll);
            BaseView.prototype.show.call(this);
            this.$el.addClass('loading');
            getSales(salesPeriod)
                .done(this.onSalesDataBind.bind(this))
                .always(function () {
                    this.$el.removeClass('loading');
                    if (this.scroll) {
                        this.scroll.scrollTo(0, 0);
                        this.scroll.refresh();
                    }
                }.bind(this));
        },

        hide: function () {
            //$(window).off("orientationHandlerCallback", this.rotateEventHandler);
            $(document).off('touchmove', preventTouchForIScroll);
            BaseView.prototype.hide.call(this);
        },

        rotateEvent: function () {
            _.each(this.charts, function (item, index) {
                // item.chart.resize();
            });
        },

        changeScopeTablet: function (e) {
            if (this.$el.hasClass('loading')) {
                e.preventDefault();
                return false;
            }
            var $target = $(e.target);

            if ($target.hasClass('active')) {
                return false;
            }
            $target.parent().find('li').removeClass('active');
            $target.addClass('active');
            this.$el.addClass('loading');
            getSales(salesPeriod = $target.data('type'))
                .done(this.onSalesDataBind.bind(this))
                .always(function () {
                    this.$el.removeClass('loading');
                    this.scroll.refresh();
                    this.scroll.scrollTo(0, 0);
                }.bind(this));
        },

        changeScopeMobile: function (e) {
            if (this.$el.hasClass('loading')) {
                e.preventDefault();
                return false;
            }
            this.$el.addClass('loading');
            getSales(salesPeriod = $(e.target).find(':selected').data('type'))
                .done(this.onSalesDataBind.bind(this))
                .always(function () {
                    this.$el.removeClass('loading');
                    this.scroll.refresh();
                    this.scroll.scrollTo(0, 0);
                }.bind(this));
        },

        onSalesDataBind: function (data) {
            salesCollection.reset();
            _.each(data, function (item) {
                salesCollection.add(_.extend(item, _.find(Consts.SalesCharts, function (chart) {
                    return chart.metricType == item.metricType;
                })));
            });
        }

    });
});